"""
Module to keep all the constants commonly used by other scripts.

Can be imported from anywhere.
"""

import pygame

# Colors
WHITE = pygame.Color("white")
BLACK = pygame.Color("black")
MIDNIGHT_BLUE = pygame.Color("midnightblue")

# Display parameters
CLOSE_BUTTON_MARGIN_TOP = 20
DEFAULT_MARGIN_TOP = 10
MARGIN_BOX = 20
MARGIN_LINKED_ELEMENT = 20
DEFAULT_POPUP_WIDTH = 400

# Standard menus and buttons sizes
BUTTON_SIZE = (200, 60)
IMAGE_BUTTON_SIZE = (250, 60)
CLOSE_BUTTON_SIZE = (150, 50)
